# Grafy

**Graf** $G$ je dvojice $G = (V, E)$, kde $V$ je konečná množina a $E \subseteq \left({V \atop 2}\right)$, přičemž
- $\left({V \atop 2}\right) = \{\{x,y\}: x,y\in V\text{ a } x\neq y\}$

je množina všech dvouprvkových množin (neuspořádaných dvojic) prvků množiny $V$.

- $V(G)$ - prvky množiny $V$ - **vrcholy** (uzly) grafu $G$
- $E(G)$ - prvky množiny $E$ - **hrany** grafu $G$

Vrcholy $x,y \in V$ jsou sousední, pokud $\{x,y\}\in E$.

### Podgraf

Mějme graf $G$, kde graf $H$ je
- podgrafem $G$, pokud platí
	- $V(H) \subseteq V(G), \quad E(H) \subseteq E(G)$
	- je to graf $G$, od kterého odebereme hrany a vrcholy
- indukovaným podgrafem $G$, pokud platí
	- $V(H) \subseteq V(G), \quad E(H) = E(G) \cap {V(H) \choose 2}$
	- graf $G$ s odebranými vrcholy a všemi hranamy k nim připojeným
### Faktor grafu

**Faktor grafu** $G$ je libovolný jeho podgraf $F$, pro který platí, že množina vrcholů $V(G) = V(F)$ a množina hran $E(G) \subseteq E(F)$. Faktor $F$ je **vlastní**, je-li různý od grafu $G$.

Řekneme, že faktor $F$ je **sudý**, má-li v něm každý vrchol sudý stupeň.

### Rovnost grafů $G_{1} = G_{2}$

Grafy $G_{1} = (V_{1}, E_{1}), G_{2} = (V_{2}, E_{2})$ jsou si rovny, pokud $V_{1} = V_{2}, E_{1} = E_{2}$

### Stupeň vrcholu

**Stupeň vrcholu** v grafu $G$ je počet hran grafu $G$, které obsahují vrchol $v$. Značí se $d_{G}(v)$.

Obvykle značíme $n = \vert V(G) \vert$ a toto číslo nazýváme **řádem** grafu $G$ (počet vrcholů), a $m = \vert E(G) \vert$ nazýváme **velikostí** grafu $G$ (počet hran).
- V grafu o $n$ vrcholech je stupeň každého vrcholu nejvýše $n-1$.
- V každém grafu platí, že $\sum_{v \in V(G)} d_{G}(v) = 2m$.
	- Důsledek: V každém grafu je počet vrcholů lichého stupně sudý.

## Neorientovaný graf

- hrany jsou definovány jako neuspořádané dvojice vrcholů
- odpovídá relaci na $V$, která je antireflexivní a symetrická

## Speciální grafy

**Biparitní (sudý) graf** $K_{m, n}$ má množinu vrcholů rozdělitelnou na dvě **disjunktní množiny** $A, B$ tak, že žádné dva **vrcholy ze stejné množiny nejsou spojeny** hranou.
- $V = A \cup B, A \cap B = \emptyset$
- $E \subseteq \{ \{a,b\} \mid a \in A, b \in B \}$

**Úplný graf** na $n$ vrcholech (značený $K_{n}$) obsahuje jako hrany všechny neuspořádané dvojice prvků $[n]$, takže $V(K_{n}) = [n], E(K_{n}) = \left({[n] \atop 2}\right)$.

**Diskrétní graf** $D_{n}$ na $n$ vrcholech nemá žádné hrany: $V(D_n) = [n], E(D_{n}) = \emptyset$.

TODO

## Homomorfizmus grafu

Nechť $G_{1} = (V_{1}, E_{1})$ a $G_{2} = (V_{2}, E_{2})$ jsou grafy. Zobrazení $f: V_{1} \to V_{2}$ je **homomorfismus**, pokud platí
1) $(x, y) \in E_{1} \implies (f(x), f(y)) \in E_{2}$,
2) $\{x, y\} \in E_{1} \implies \{f(x), f(y)\} \in E_{2}$.
- každá hrana se zobrazí na hranu
- zkráceně píšeme $f: G_{1} \to G_{2}$

Poznámka: $f: V_{1} \to V_{2}$ je homomorfizmus právě když $e \in E_{1} \implies f^*(e) \in E_{2}$.
### Zobrazení indukované zobrazením

Nechť $f: V_{1} \to V_{2}$ je **homomorfizmus**. Potom zobrazení $f^*: \left({V_{1} \atop 2}\right) \to \left({V_{2} \atop 2}\right)$ definované vztahy
1) $f^*((u, v)) = (f(u), f(v))$,
2) $f^*(\{u, v\}) = \{f(u), f(v)\}$

nazveme **zobrazení indukované zobrazením** $f$.

### Další morfizmy

Nechť $G_{1} = (V_{1}, E_{1})$ a $G_{2} = (V_{2}, E_{2})$ jsou grafy a zobrazení $f: V_{1} \to V_{2}$ je **homomorfismus**. Potom se $f$ nazývá
1) **vrcholový monomorfizmus**, je-li $f$ prosté,
2) **vrcholový epimorfizmus**, je-li $f$ na,
3) **hranový monomorfizmus**, je-li $f^*$ prosté,
4) **hranový epimorfizmus**, je-li $f^*$ na,
5) **monomorfizmus**, jsou-li $f$ i $f^*$ prostá,
6) **epimorfizmus**, jsou-li $f$ i $f^*$ na,
7) **izomorfizmus**, jsou-li $f$ i $f^*$ zároveň prostá i na.

*(mono = prosté, epi = zobrazení na)*

Grafy $G_{1}, G_{2}$ jsou **izomorfní**, jestliže existuje izomorfizmus $G_{1}$ na $G_{2}$ a píšeme $G_{1} \simeq G_{2}$

### Automorfismus grafu

**Automorfismem grafu** $G$ nazveme izomorfizmus $G \to G$.

Izomorfismus může být **triviální** (identické zobrazení, $v_{i} \to v_{i}, \dots$) nebo **netriviální**. Složení izomorfismů je opět izomorfismus.

Množina automorfismů grafu $G$ s operací skládání tvoří grupu a značí se $\text{Aut}(G)$.

# Orientované grafy

- **Orientovaný graf** je dvojice $G = (V, E)$, kde $V$ je množina vrcholů a $E \subseteq V \times V$ je množina hran. (hrany jsou nyní prvky kartézského součinu, tedy uspořádané dvojice vrcholů)
- orientované grafy odpovídají **binárním relacím**
- graf může obsahovat **dvojici protichůdných hran**
- má upravené definice některých pojmů